#version 130
precision mediump float;

uniform float shadow;
uniform float omnidirectional_range;
uniform vec4 giantBananaLight_position;
uniform vec4 giantBananaLight_color;
uniform vec3 sunLight_position;
uniform vec4 sunLight_color;
uniform vec4 ambient_color;
uniform vec4 specular_color;
uniform vec4 camera_position;
uniform vec4 fog_color;
uniform float fog_density;
uniform sampler2D texture;
uniform sampler2D shadowMapPositiveX;
uniform sampler2D shadowMapNegativeX;
uniform sampler2D shadowMapPositiveY;
uniform sampler2D shadowMapNegativeY;
uniform sampler2D shadowMapPositiveZ;
uniform sampler2D shadowMapNegativeZ;

in vec2 frag_texCoord;
in vec4 texCoordPositiveX;
in vec4 texCoordNegativeX;
in vec4 texCoordPositiveY;
in vec4 texCoordNegativeY;
in vec4 texCoordPositiveZ;
in vec4 texCoordNegativeZ;
in vec3 world_normal;
in vec4 world_position;

out vec4 fragColor;

void main() {
	if (shadow <= 0.0) {
		vec3 normal = normalize(world_normal);
		vec3 position = world_position.xyz / world_position.w;
		vec3 view_vector = normalize(camera_position.xyz - position);
		vec4 frag_Color = texture2D(texture, frag_texCoord);
		vec3 sunLight_dir = normalize(sunLight_position - position);

		vec4 ambient = ambient_color * frag_Color;	
		vec4 diffuse = frag_Color * sunLight_color * max(0.0, dot(normal, sunLight_dir));
		vec4 specular = specular_color * pow(max(0.0, dot(normalize(reflect(-sunLight_dir, normal)), view_vector)), 1.0) * 0.35;
	
		vec3 giantBananaLight_dir = giantBananaLight_position.xyz - position;
		float distanceToGiantBanana = length(giantBananaLight_dir);
	
		if (distanceToGiantBanana < omnidirectional_range) {
			vec3 texCoordPositiveX_normalized = texCoordPositiveX.xyz / texCoordPositiveX.w;
			vec3 texCoordNegativeX_normalized = texCoordNegativeX.xyz / texCoordNegativeX.w;
			vec3 texCoordPositiveY_normalized = texCoordPositiveY.xyz / texCoordPositiveY.w;
			vec3 texCoordNegativeY_normalized = texCoordNegativeY.xyz / texCoordNegativeY.w;
			vec3 texCoordPositiveZ_normalized = texCoordPositiveZ.xyz / texCoordPositiveZ.w;
			vec3 texCoordNegativeZ_normalized = texCoordNegativeZ.xyz / texCoordNegativeZ.w;
	
			float shadowFactor = 0.0;
		
			float depthPositiveX = texture2D(shadowMapPositiveX, texCoordPositiveX_normalized.xy).x;
			float depthNegativeX = texture2D(shadowMapNegativeX, texCoordNegativeX_normalized.xy).x;
			float depthPositiveY = texture2D(shadowMapPositiveY, texCoordPositiveY_normalized.xy).x;
			float depthNegativeY = texture2D(shadowMapNegativeY, texCoordNegativeY_normalized.xy).x;
			float depthPositiveZ = texture2D(shadowMapPositiveZ, texCoordPositiveZ_normalized.xy).x;
			float depthNegativeZ = texture2D(shadowMapNegativeZ, texCoordNegativeZ_normalized.xy).x;
		
			if ((depthPositiveX >= texCoordPositiveX_normalized.z) ||
				(depthNegativeX >= texCoordNegativeX_normalized.z) ||
				(depthPositiveY >= texCoordPositiveY_normalized.z) ||
				(depthNegativeY >= texCoordNegativeY_normalized.z) ||
				(depthPositiveZ >= texCoordPositiveZ_normalized.z) ||
				(depthNegativeZ >= texCoordNegativeZ_normalized.z)) {
				shadowFactor = 1.0;
			}
			
			giantBananaLight_dir = normalize(giantBananaLight_dir);
	
			//wir verwenden nur lineare Attenuation
			float attenuation = (omnidirectional_range - distanceToGiantBanana) / omnidirectional_range;
			diffuse = diffuse +	frag_Color * max(0.0, dot(normal, giantBananaLight_dir)) * giantBananaLight_color * attenuation * shadowFactor;
		}
	
		float z = gl_FragCoord.z / gl_FragCoord.w;
		float fog_factor = exp2((-1.0) * pow((fog_density * z), 2.0));

		fragColor = ambient + diffuse + specular;
		fragColor = mix(fog_color, fragColor, fog_factor);
		fragColor.w = 1.0;
	}
	else {
		fragColor = vec4(0.0, 0.0, 1.0, 1.0);
	}
}