#version 130
precision mediump float;

uniform int samples;
uniform float shadow;
uniform float omnidirectional_range;
uniform vec4 giantBananaLight_position;
uniform vec4 giantBananaLight_color;
uniform vec3 sunLight_position;
uniform vec4 sunLight_color;
uniform vec4 ambient_color;
uniform vec4 specular_color;
uniform vec4 camera_position;
uniform vec4 fog_color;
uniform float fog_density;
uniform sampler2D noise; 
uniform sampler2D normal;
uniform sampler2D shadowMapPositiveX;
uniform sampler2D shadowMapNegativeX;
uniform sampler2D shadowMapNegativeY;
uniform sampler2D shadowMapPositiveZ;
uniform sampler2D shadowMapNegativeZ;

in vec2 frag_texCoord;
in vec4 texCoordPositiveX;
in vec4 texCoordNegativeX;
in vec4 texCoordNegativeY;
in vec4 texCoordPositiveZ;
in vec4 texCoordNegativeZ;
in vec3 world_normal;
in vec4 world_position;

out vec4 fragColor;

void main() {
	if (shadow < 1.0) {
		float grain_weight = 0.15;
		vec3 position = world_position.xyz / world_position.w;
		vec3 view_vector = normalize(camera_position.xyz - position);
		vec3 sunLight_dir = normalize(sunLight_position - position);
	
		vec3 normal_by_map = texture2D(normal, frag_texCoord * 2.0).xyz;
		normal_by_map += texture2D(normal, frag_texCoord * 4.0).xyz;
		normal_by_map += texture2D(normal, frag_texCoord * 8.0).xyz;
		normal_by_map += texture2D(normal, frag_texCoord * 16.0).xyz;
		normal_by_map += texture2D(normal, frag_texCoord * 64.0).xyz;
		normal_by_map += texture2D(normal, frag_texCoord * 96.0).xyz;
		normal_by_map += texture2D(normal, frag_texCoord * 1500.0).xyz * 2;
		normal_by_map += texture2D(normal, frag_texCoord * 2048.0).xyz * 2;
		normal_by_map = normal_by_map / 10.0;
	
		vec3 normal_new = normalize(mix(world_normal, normalize(world_normal + normal_by_map), pow(world_normal.y, 2.0)));
	
		vec4 grain_large = texture2D(noise, frag_texCoord * 64.0f);
		vec4 grain_small = texture2D(noise, frag_texCoord * 512.0f);
		vec4 grain_very_small = texture2D(noise, frag_texCoord * 4000.0f) * 2.0;
		vec4 grain_mixed = mix(grain_large, grain_small, pow(normal_new.y, 0.5));

		vec4 ambient = ambient_color;
		vec4 diffuse = max(0.0, dot(normal_new, sunLight_dir)) + max(0.0, dot(normal_new, normalize(sunLight_dir + normal_new))) * sunLight_color;
		vec4 specular = pow(max(0.0, dot(normalize(reflect(-sunLight_dir, normal_new)), view_vector)), 4.0) * specular_color;
	
		vec3 giantBananaLight_dir = giantBananaLight_position.xyz - position;
		float distanceToGiantBanana = length(giantBananaLight_dir);
	
		if (distanceToGiantBanana < omnidirectional_range) {
			vec3 texCoordPositiveX_normalized = texCoordPositiveX.xyz / texCoordPositiveX.w;
			vec3 texCoordNegativeX_normalized = texCoordNegativeX.xyz / texCoordNegativeX.w;
			vec3 texCoordNegativeY_normalized = texCoordNegativeY.xyz / texCoordNegativeY.w;
			vec3 texCoordPositiveZ_normalized = texCoordPositiveZ.xyz / texCoordPositiveZ.w;
			vec3 texCoordNegativeZ_normalized = texCoordNegativeZ.xyz / texCoordNegativeZ.w;
	
			float shadowFactor = 0.0;
		
			vec2 offset[8];
			offset[0] = vec2(0, 0);
			offset[1] = vec2(0.002942, 0.0039906);
			offset[2] = vec2(-0.0029455, 0.0067689);
			offset[3] = vec2(0.0009418, -0.0042938);
			offset[4] = vec2(0.0014495, 0.00029387);
			offset[5] = vec2(-0.00091588, -0.00045771);
			offset[6] = vec2(-0.0013154, -0.0013912);
			offset[7] = vec2(-0.00088277, 0.00077676);
		
			float depthPositiveX;
			float depthNegativeX;
			float depthNegativeY;
			float depthPositiveZ;
			float depthNegativeZ;
		
			for (int i = 0; i < samples; i++) {
				depthPositiveX = texture2D(shadowMapPositiveX, texCoordPositiveX_normalized.xy + offset[i]).x;
				depthNegativeX = texture2D(shadowMapNegativeX, texCoordNegativeX_normalized.xy + offset[i]).x;
				depthNegativeY = texture2D(shadowMapNegativeY, texCoordNegativeY_normalized.xy + offset[i]).x;
				depthPositiveZ = texture2D(shadowMapPositiveZ, texCoordPositiveZ_normalized.xy + offset[i]).x;
				depthNegativeZ = texture2D(shadowMapNegativeZ, texCoordNegativeZ_normalized.xy + offset[i]).x;
		
				if ((depthPositiveX >= texCoordPositiveX_normalized.z) ||
					(depthNegativeX >= texCoordNegativeX_normalized.z) ||
					(depthNegativeY >= texCoordNegativeY_normalized.z) ||
					(depthPositiveZ >= texCoordPositiveZ_normalized.z) ||
					(depthNegativeZ >= texCoordNegativeZ_normalized.z)) {
					shadowFactor += 1.0 / samples;
				}
			}
			
			if (shadowFactor > 0.0) {
				giantBananaLight_dir = normalize(giantBananaLight_dir);
			
				//wir verwenden nur lineare Attenuation
				float attenuation = (omnidirectional_range - distanceToGiantBanana) / omnidirectional_range;
				diffuse = diffuse +	max(0.0, dot(normal_new, normalize(giantBananaLight_dir + normal_new))) * giantBananaLight_color * attenuation * shadowFactor;
			}
		}
	
		vec4 color = mix(sunLight_color + grain_mixed * grain_weight, vec4(0.75, 0.8, 0.8, 0.0), pow(normal_new.y, 1.5));
	
		float z = gl_FragCoord.z / gl_FragCoord.w;
		float fog_factor = exp2((-1.0) * pow((fog_density * z), 2.0));
	
		fragColor = ambient;
		fragColor += grain_mixed * grain_weight;
		fragColor += diffuse;
		fragColor *= color;
		fragColor += specular * grain_mixed * grain_very_small;
		fragColor = mix(fragColor, vec4(0.75, 0.8, 0.8, 1.0), 0.4);
		fragColor = mix(fog_color, fragColor, fog_factor);
		fragColor.w = 1.0;
	}
	else {
		fragColor = vec4(0.2, 0.2, 0.2, 1.0);
	}
}