#version 130
precision mediump float;

uniform float shadow;
uniform float omnidirectional_range;
uniform vec4 giantBananaLight_position;
uniform vec4 giantBananaLight_color;
uniform vec3 sunLight_position;
uniform vec4 sunLight_color;
uniform vec4 ambient_color;
uniform vec4 specular_color;
uniform vec4 camera_position;
uniform vec4 fog_color;
uniform float fog_density;
uniform sampler2D texture;

in vec2 frag_texCoord;
in vec3 world_normal;
in vec4 world_position;

out vec4 fragColor;

void main() {
	if (shadow < 1.0) {
		vec3 normal = normalize(world_normal);
		vec3 position = world_position.xyz / world_position.w;
		vec3 view_vector = normalize(camera_position.xyz - position);
		vec4 frag_Color = texture2D(texture, frag_texCoord);
		vec3 sunLight_dir = normalize(sunLight_position - position);

		vec4 ambient = ambient_color * frag_Color;	
		vec4 diffuse = frag_Color * sunLight_color * max(0.0, dot(normal, sunLight_dir));
		vec4 specular = specular_color * pow(max(0.0, dot(normalize(reflect(-sunLight_dir, normal)), view_vector)), 1.0) * 0.35;
	
		vec3 giantBananaLight_dir = giantBananaLight_position.xyz - position;
		float distanceToGiantBanana = length(giantBananaLight_dir);
	
		if (distanceToGiantBanana < omnidirectional_range) {
			giantBananaLight_dir = normalize(giantBananaLight_dir);
	
			//wir verwenden nur lineare Attenuation 1 / distance
			diffuse = diffuse +	frag_Color * giantBananaLight_color * max(0.0, dot(normal, giantBananaLight_dir)) / distanceToGiantBanana;
		}
	
		float z = gl_FragCoord.z / gl_FragCoord.w;
		float fog_factor = exp2((-1.0) * pow((fog_density * z), 2.0));

		fragColor = ambient + diffuse + specular;
		fragColor = mix(fog_color, fragColor, fog_factor);
		fragColor.w = 1.0;
	}
	else {
		fragColor = vec4(0.0, 1.0, 0.0, 1.0);
	}
}